@@ -12,15 +12,22 @@
 #'   row.
 #' @param asp Aspect ratio (width/height) of the target canvas. This is used to convert
 #'   widths to heights and vice versa for rotated boxes
+#' @keywords internal
 #' @export
 clip_lines <- function(x, y, id, clip_boxes, asp = 1) {
   out = list(x = x, y = y, id = id)
   for (i in 1:nrow(clip_boxes)) {
     box <- clip_boxes[i, ]
     out <- clip_lines_impl(
-      out$x, out$y, out$id,
-      box$x, box$y, box$width, box$height, box$theta,
-      asp
+      as.double(out$x),
+      as.double(out$y),
+      as.integer(out$id),
+      as.double(box$x),
+      as.double(box$y),
+      as.double(box$width),
+      as.double(box$height),
+      as.double(box$theta),
+      as.double(asp)
     )
   }
 
