@@ -1,13 +1,15 @@
-#include <Rcpp.h>
-using namespace Rcpp;
+#include "cpp11/doubles.hpp"
+#include "cpp11/integers.hpp"
+#include "cpp11/list.hpp"
+#define R_NO_REMAP
 
-#include <iostream>
 #include <cmath>
 #include <cstddef>  // for size_t
 using namespace std;
 
 #include "clip-lines.h"
 
+using namespace cpp11::literals;
 
 // calculates the intersection point of a line segment and
 // the unit box, assuming p1 is outside and p2 is inside.
@@ -246,7 +248,7 @@ bool in_unit_box(const point &p) {
 }
 
 // helper function for crop_lines()
-void record_points(NumericVector &x_out, NumericVector &y_out, IntegerVector &id_out,
+void record_points(cpp11::writable::doubles &x_out, cpp11::writable::doubles &y_out, cpp11::writable::integers &id_out,
                    const point &p1, const point &p2, int &cur_id_out,
                    bool &p1_recorded, bool &p2_recorded, bool &new_line_segment) {
   if (new_line_segment) {
@@ -289,27 +291,51 @@ void record_points(NumericVector &x_out, NumericVector &y_out, IntegerVector &id
 // @param asp Aspect ratio (width/height) of the target canvas. This is used to convert widths
 //  to heights and vice versa for rotated boxes
 // @export
-// [[Rcpp::export]]
-List clip_lines_impl(const NumericVector &x, const NumericVector &y, const IntegerVector &id,
-                     const double p_mid_x, const double p_mid_y, const double width,
-                     const double height, const double theta, const double asp = 1) {
-  // output variables
-  NumericVector x_out, y_out;
-  IntegerVector id_out;
+[[cpp11::register]]
+cpp11::writable::list clip_lines_impl(
+  cpp11::doubles x,
+  cpp11::doubles y,
+  cpp11::integers id,
+  double p_mid_x,
+  double p_mid_y,
+  double width,
+  double height,
+  double theta,
+  double asp
+) {
+  // input
+  int n = x.size();
+  double* x_p = REAL(x);
+  double* y_p = REAL(y);
+  int* id_p = INTEGER(id);
+
+  // output variable
+  //SEXP res = PROTECT(Rf_allocVector(VECSXP, 3));
+  //SEXP names = PROTECT(Rf_allocVector(STRSXP, 3));
+  //SET_STRING_ELT(names, 0, Rf_mkChar("x"));
+  //SET_STRING_ELT(names, 1, Rf_mkChar("y"));
+  //SET_STRING_ELT(names, 2, Rf_mkChar("id"));
+  //Rf_setAttrib(res, Rf_install("names"), names);
+
+  cpp11::writable::doubles x_out, y_out;
+  cpp11::writable::integers id_out;
 
   // input checks
-  if (x.size() != y.size()) {
-    stop("Number of x and y coordinates must match.");
+  if (n != y.size()) {
+    cpp11::stop("Number of x and y coordinates must match.");
   }
-  if (x.size() != id.size()) {
-    stop("Number of x coordinates and id values must match.");
+  if (n != id.size()) {
+    cpp11::stop("Number of x coordinates and id values must match.");
   }
-  if (x.size() == 0) {
+  if (n == 0) {
     // empty input, return empty output
-    return List::create(_["x"] = x_out, _["y"] = y_out, _["id"] = id_out);
+    return cpp11::writable::list({
+      "x"_nm = x_out,
+      "y"_nm = y_out,
+      "id"_nm = id_out
+    });
   }
 
-
   // set up transformation
   // lower left point of cropping rectangle
   point ll(p_mid_x - width*cos(theta)/2 + (height/asp)*sin(theta)/2,
@@ -322,35 +348,35 @@ List clip_lines_impl(const NumericVector &x, const NumericVector &y, const Integ
   unitbox_transformer t(ll, lr, ul);
 
   // crop
-  int cur_id = id[0];
+  int cur_id = id_p[0];
   int cur_id_out = 0; // first output id - 1
   point p1, p2, p1t, p2t;
   point crop1, crop2;
-  p1 = point(x[0], y[0]);
+  p1 = point(x_p[0], y_p[0]);
   p1t = t.transform(p1);
 
   bool p1_recorded = in_unit_box(p1t); // record only if not in unit box, catches singlets
   bool p2_recorded = true; // when we first enter the loop, have only p1 unrecorded
   bool new_line_segment = true;
 
-  size_t i = 1;
-  while(i < x.size()) {
-    if (cur_id != id[i]) {
+  int i = 1;
+  while(i < n) {
+    if (cur_id != id_p[i]) {
       // id mismatch means we are starting a new line segment
 
       // first record any points that haven't been recorded yet. catches singlets
       record_points(x_out, y_out, id_out, p1, p2, cur_id_out,
                     p1_recorded, p2_recorded, new_line_segment);
       // now set up next line segment
-      p1 = point(x[i], y[i]);
+      p1 = point(x_p[i], y_p[i]);
       p1t = t.transform(p1);
-      cur_id = id[i];
+      cur_id = id_p[i];
       p1_recorded = in_unit_box(p1t); // record only if not in unit box, catches singlets
       new_line_segment = true;
       i++;
       continue;
     }
-    p2 = point(x[i], y[i]);
+    p2 = point(x_p[i], y_p[i]);
     p2t = t.transform(p2);
     p2_recorded = false;
     segment_crop_type result = crop_to_unit_box(p1t, p2t, crop1, crop2);
@@ -398,7 +424,11 @@ List clip_lines_impl(const NumericVector &x, const NumericVector &y, const Integ
   record_points(x_out, y_out, id_out, p1, p2, cur_id_out,
                 p1_recorded, p2_recorded, new_line_segment);
 
-  return List::create(_["x"] = x_out, _["y"] = y_out, _["id"] = id_out);
+  return cpp11::writable::list({
+    "x"_nm = x_out,
+    "y"_nm = y_out,
+    "id"_nm = id_out
+  });
 }
 
 
