@@ -3,15 +3,20 @@
 // Includes merging of line segments and polygons.
 // Written by Claus O. Wilke
 
-// [[Rcpp::plugins(cpp11)]]
-#include <Rcpp.h>
-using namespace Rcpp;
+#include "cpp11/data_frame.hpp"
+#include "cpp11/doubles.hpp"
+#include "cpp11/integers.hpp"
+#include "cpp11/list.hpp"
+#include "cpp11/matrix.hpp"
+#include "cpp11/protect.hpp"
+#define R_NO_REMAP
 
 #include <iostream>
 #include <vector>
 #include <unordered_map>
 
 using namespace std;
+using namespace cpp11::literals;
 
 #include "polygon.h" // for point
 
@@ -77,8 +82,9 @@ ostream & operator<<(ostream &out, const point_connect &pc) {
 class isobander {
 protected:
   int nrow, ncol; // numbers of rows and columns
-  const NumericVector &grid_x,&grid_y;
-  const NumericMatrix &grid_z;
+  cpp11::doubles grid_x, grid_y;
+  cpp11::doubles_matrix<> grid_z;
+  double *grid_x_p, *grid_y_p, *grid_z_p;
   double vlo, vhi; // low and high cutoff values
   grid_point tmp_poly[8]; // temp storage for elementary polygons; none has more than 8 vertices
   point_connect tmp_point_connect[8];
@@ -87,6 +93,8 @@ protected:
   typedef unordered_map<grid_point, point_connect, grid_point_hasher> gridmap;
   gridmap polygon_grid;
 
+  bool interrupted;
+
   void reset_grid() {
     polygon_grid.clear();
 
@@ -98,7 +106,7 @@ protected:
   // internal member functions
 
   double central_value(int r, int c) {// calculates the central value of a given cell
-    return (grid_z(r, c) + grid_z(r, c + 1) + grid_z(r + 1, c) + grid_z(r + 1, c + 1))/4;
+    return (grid_z_p[r + c * nrow] + grid_z_p[r + (c + 1) * nrow] + grid_z_p[r + 1 + c * nrow] + grid_z_p[r + 1 + (c + 1) * nrow])/4;
   }
 
   void poly_start(int r, int c, point_type type) { // start a new elementary polygon
@@ -213,7 +221,7 @@ protected:
             tmp_point_connect[i].altpoint = true;
             break;
           default:
-            stop("undefined merging configuration: %i\n", score);
+            cpp11::stop("undefined merging configuration: %i\n", score);
           }
         }
       }
@@ -257,33 +265,36 @@ protected:
   point calc_point_coords(const grid_point &p) {
     switch(p.type) {
     case grid:
-      return point(grid_x[p.c], grid_y[p.r]);
+      return point(grid_x_p[p.c], grid_y_p[p.r]);
     case hintersect_lo: // intersection with horizontal edge, low value
-      return point(interpolate(grid_x[p.c], grid_x[p.c+1], grid_z(p.r, p.c), grid_z(p.r, p.c+1), vlo), grid_y[p.r]);
+      return point(interpolate(grid_x_p[p.c], grid_x_p[p.c+1], grid_z_p[p.r + p.c * nrow], grid_z_p[p.r + (p.c + 1) * nrow], vlo), grid_y_p[p.r]);
     case hintersect_hi: // intersection with horizontal edge, high value
-      return point(interpolate(grid_x[p.c], grid_x[p.c+1], grid_z(p.r, p.c), grid_z(p.r, p.c+1), vhi), grid_y[p.r]);
+      return point(interpolate(grid_x_p[p.c], grid_x_p[p.c+1], grid_z_p[p.r + p.c * nrow], grid_z_p[p.r + (p.c + 1) * nrow], vhi), grid_y_p[p.r]);
     case vintersect_lo: // intersection with vertical edge, low value
-      return point(grid_x[p.c], interpolate(grid_y[p.r], grid_y[p.r+1], grid_z(p.r, p.c), grid_z(p.r+1, p.c), vlo));
+      return point(grid_x_p[p.c], interpolate(grid_y_p[p.r], grid_y_p[p.r+1], grid_z_p[p.r + p.c * nrow], grid_z_p[p.r + 1 + p.c * nrow], vlo));
     case vintersect_hi: // intersection with vertical edge, high value
-      return point(grid_x[p.c], interpolate(grid_y[p.r], grid_y[p.r+1], grid_z(p.r, p.c), grid_z(p.r+1, p.c), vhi));
+      return point(grid_x_p[p.c], interpolate(grid_y_p[p.r], grid_y_p[p.r+1], grid_z_p[p.r + p.c * nrow], grid_z_p[p.r + 1 + p.c * nrow], vhi));
     default:
       return point(0, 0); // should never get here
     }
   }
 
 public:
-  isobander(const NumericVector &x, const NumericVector &y, const NumericMatrix &z, double value_low = 0, double value_high = 0) :
-    grid_x(x), grid_y(y), grid_z(z), vlo(value_low), vhi(value_high)
+  isobander(cpp11::doubles x, cpp11::doubles y, cpp11::doubles_matrix<> z, double value_low = 0, double value_high = 0) :
+    grid_x(x), grid_y(y), grid_z(z), grid_x_p(REAL(x)), grid_y_p(REAL(y)),
+    grid_z_p(REAL(z)), vlo(value_low), vhi(value_high), interrupted(false)
   {
     nrow = grid_z.nrow();
     ncol = grid_z.ncol();
 
-    if (grid_x.size() != ncol) {stop("Number of x coordinates must match number of columns in density matrix.");}
-    if (grid_y.size() != nrow) {stop("Number of y coordinates must match number of rows in density matrix.");}
+    if (grid_x.size() != ncol) {cpp11::stop("Number of x coordinates must match number of columns in density matrix.");}
+    if (grid_y.size() != nrow) {cpp11::stop("Number of y coordinates must match number of rows in density matrix.");}
   }
 
   virtual ~isobander() {}
 
+  bool was_interrupted() {return interrupted;}
+
   void set_value(double value_low, double value_high) {
     vlo = value_low;
     vhi = value_high;
@@ -294,38 +305,37 @@ public:
     reset_grid();
 
     // setup matrix of ternarized cell representations
-    IntegerVector v(nrow*ncol);
-    IntegerVector::iterator iv = v.begin();
-    for (NumericMatrix::const_iterator it = grid_z.begin(); it != grid_z.end(); it++) {
-      *iv = (*it >= vlo && *it < vhi) + 2*(*it >= vhi);
+    vector<int> ternarized(nrow*ncol);
+    vector<int>::iterator iv = ternarized.begin();
+    for (int i = 0; i < nrow * ncol; ++i) {
+      *iv = (grid_z_p[i] >= vlo && grid_z_p[i] < vhi) + 2*(grid_z_p[i] >= vhi);
       iv++;
     }
 
-    IntegerMatrix ternarized(nrow, ncol, v.begin());
-    IntegerMatrix cells(nrow - 1, ncol - 1);
+    vector<int> cells((nrow - 1) * (ncol - 1));
 
     for (int r = 0; r < nrow-1; r++) {
       for (int c = 0; c < ncol-1; c++) {
         int index;
-        if (NumericMatrix::is_na(grid_z(r, c)) || NumericMatrix::is_na(grid_z(r, c + 1)) ||
-            NumericMatrix::is_na(grid_z(r + 1, c)) || NumericMatrix::is_na(grid_z(r + 1, c + 1))) {
+        if (!R_finite(grid_z_p[r + c * nrow]) || !R_finite(grid_z_p[r + (c + 1) * nrow]) ||
+            !R_finite(grid_z_p[r + 1 + c * nrow]) || !R_finite(grid_z_p[r + 1 + (c + 1) * nrow])) {
           // we don't draw any contours if at least one of the corners is NA
           index = 0;
         } else {
-          index = 27*ternarized(r, c) + 9*ternarized(r, c + 1) + 3*ternarized(r + 1, c + 1) + ternarized(r + 1, c);
+          index = 27*ternarized[r + c * nrow] + 9*ternarized[r + (c + 1) * nrow] + 3*ternarized[r + 1 + (c + 1) * nrow] + ternarized[r + 1 + c * nrow];
         }
-        cells(r, c) = index;
+        cells[r + c * (nrow - 1)] = index;
         //cout << index << " ";
       }
       //cout << endl;
     }
-    checkUserInterrupt();
+    cpp11::check_user_interrupt();
 
     // all polygons must be drawn clockwise for proper merging
     for (int r = 0; r < nrow-1; r++) {
       for (int c = 0; c < ncol-1; c++) {
         //cout << r << " " << c << " " << cells(r, c) << endl;
-        switch(cells(r, c)) {
+        switch(cells[r + c * (nrow - 1)]) {
         // doing cases out of order, sorted by type, is easier to keep track of
 
         // no contour
@@ -1217,9 +1227,9 @@ public:
     }
   }
 
-  virtual List collect() {
-    // make polygons
-    vector<double> x_out, y_out; vector<int> id;  // vectors holding resulting polygon paths
+  virtual cpp11::writable::list collect() {// make polygons
+    cpp11::writable::doubles x_out, y_out;
+    cpp11::writable::integers id;  // vectors holding resulting polygon paths
     int cur_id = 0;           // id counter for the polygon lines
 
     // iterate over all locations in the polygon grid
@@ -1264,11 +1274,17 @@ public:
           cur = newcur;
         }
         i++;
-        if (i % 100000 == 0)
-          checkUserInterrupt();
+        if (i % 100000 == 0) {
+          cpp11::check_user_interrupt();
+        }
       } while (!(cur == start)); // keep going until we reach the start point again
     }
-    return List::create(_["x"] = x_out, _["y"] = y_out, _["id"] = id);
+
+    return cpp11::writable::list({
+      "x"_nm = x_out,
+      "y"_nm = y_out,
+      "id"_nm = id
+    });
   }
 };
 
@@ -1311,7 +1327,7 @@ protected:
         polygon_grid[tmp_poly[1]].next = tmp_poly[0];
       } else {
         // should never go here
-        stop("cannot merge line segment at interior of existing line segment");
+        cpp11::stop("cannot merge line segment at interior of existing line segment");
       }
       break;
     case 2: // only second point connects
@@ -1323,7 +1339,7 @@ protected:
         polygon_grid[tmp_poly[0]].next = tmp_poly[1];
       } else {
         // should never go here
-        stop("cannot merge line segment at interior of existing line segment");
+        cpp11::stop("cannot merge line segment at interior of existing line segment");
       }
       break;
     case 3: // two-way merge
@@ -1359,8 +1375,9 @@ protected:
               polygon_grid[cur].next = tmp;
               cur = tmp;
               i++;
-              if (i % 100000 == 0)
-                checkUserInterrupt();
+              if (i % 100000 == 0) {
+                cpp11::check_user_interrupt();
+              }
             } while (!(cur == grid_point()));
           }
           break;
@@ -1378,18 +1395,19 @@ protected:
               polygon_grid[cur].prev = tmp;
               cur = tmp;
               i++;
-              if (i % 100000 == 0)
-                checkUserInterrupt();
+              if (i % 100000 == 0) {
+                cpp11::check_user_interrupt();
+              }
             } while (!(cur == grid_point()));
           }
           break;
         default:  // should never go here
-          stop("cannot merge line segment at interior of existing line segment");
+          cpp11::stop("cannot merge line segment at interior of existing line segment");
         }
       }
     break;
     default:
-      stop("unknown merge state");
+      cpp11::stop("unknown merge state");
     }
 
     //cout << "new grid:" << endl;
@@ -1397,7 +1415,7 @@ protected:
   }
 
 public:
-  isoliner(const NumericVector &x, const NumericVector &y, const NumericMatrix &z, double value = 0) :
+  isoliner(cpp11::doubles x, cpp11::doubles y, cpp11::doubles_matrix<> z, double value = 0) :
     isobander(x, y, z, value, 0) {}
 
   void set_value(double value) {
@@ -1409,25 +1427,24 @@ public:
     reset_grid();
 
     // setup matrix of binarized cell representations
-    IntegerVector v(nrow*ncol);
-    IntegerVector::iterator iv = v.begin();
-    for (NumericMatrix::const_iterator it = grid_z.begin(); it != grid_z.end(); it++) {
-      *iv = (*it >= vlo);
+    vector<int> binarized(nrow*ncol);
+    vector<int>::iterator iv = binarized.begin();
+    for (int i = 0; i < nrow * ncol; ++i) {
+      *iv = (grid_z_p[i] >= vlo);
       iv++;
     }
 
-    IntegerMatrix binarized(nrow, ncol, v.begin());
-    IntegerMatrix cells(nrow - 1, ncol - 1);
+    vector<int> cells((nrow - 1) * (ncol - 1));
 
     for (int r = 0; r < nrow-1; r++) {
       for (int c = 0; c < ncol-1; c++) {
         int index;
-        if (NumericMatrix::is_na(grid_z(r, c)) || NumericMatrix::is_na(grid_z(r, c + 1)) ||
-            NumericMatrix::is_na(grid_z(r + 1, c)) || NumericMatrix::is_na(grid_z(r + 1, c + 1))) {
-          // we don't draw any contour lines if at least one of the corners is NA
+        if (!R_finite(grid_z_p[r + c * nrow]) || !R_finite(grid_z_p[r + (c + 1) * nrow]) ||
+            !R_finite(grid_z_p[r + 1 + c * nrow]) || !R_finite(grid_z_p[r + 1 + (c + 1) * nrow])) {
+          // we don't draw any contours if at least one of the corners is NA
           index = 0;
         } else {
-          index = 8*binarized(r, c) + 4*binarized(r, c + 1) + 2*binarized(r+1, c+1) + 1*binarized(r + 1, c);
+          index = 8*binarized[r + c * nrow] + 4*binarized[r + (c + 1) * nrow] + 2*binarized[r + 1 + (c + 1) * nrow] + 1*binarized[r + 1 + c * nrow];
         }
 
         // two-segment saddles
@@ -1437,15 +1454,15 @@ public:
           index = 5;
         }
 
-        cells(r, c) = index;
+        cells[r + c * (nrow - 1)] = index;
       }
     }
 
-    checkUserInterrupt();
+    cpp11::check_user_interrupt();
 
     for (int r = 0; r < nrow-1; r++) {
       for (int c = 0; c < ncol-1; c++) {
-        switch(cells(r, c)) {
+        switch(cells[r + c * (nrow - 1)]) {
         case 0: break;
         case 1:
           line_start(r, c, vintersect_lo);
@@ -1533,9 +1550,10 @@ public:
     }
   }
 
-  virtual List collect() {
+  virtual cpp11::writable::list collect() {
     // make line segments
-    vector<double> x_out, y_out; vector<int> id;  // vectors holding resulting polygon paths
+    cpp11::writable::doubles x_out, y_out;
+    cpp11::writable::integers id;  // vectors holding resulting polygon paths
     int cur_id = 0;           // id counter for individual line segments
 
     // iterate over all locations in the polygon grid
@@ -1557,8 +1575,9 @@ public:
         do {
           cur = polygon_grid[cur].prev;
           i++;
-          if (i % 100000 == 0)
-            checkUserInterrupt();
+          if (i % 100000 == 0) {
+            cpp11::check_user_interrupt();
+          }
         } while (!(cur == start || polygon_grid[cur].prev == grid_point()));
       }
 
@@ -1567,6 +1586,7 @@ public:
       do {
         //cout << cur << endl;
         point p = calc_point_coords(cur);
+
         x_out.push_back(p.x);
         y_out.push_back(p.y);
         id.push_back(cur_id);
@@ -1575,8 +1595,9 @@ public:
         polygon_grid[cur].collected = true;
         cur = polygon_grid[cur].next;
         i++;
-        if (i % 100000 == 0)
-          checkUserInterrupt();
+        if (i % 100000 == 0) {
+          cpp11::check_user_interrupt();
+        }
       } while (!(cur == start || cur == grid_point())); // keep going until we reach the start point again
       // if we're back to start, need to output that point one more time
       if (cur == start) {
@@ -1586,40 +1607,46 @@ public:
         id.push_back(cur_id);
       }
     }
-    return List::create(_["x"] = x_out, _["y"] = y_out, _["id"] = id);
+    return cpp11::writable::list({
+      "x"_nm = x_out,
+      "y"_nm = y_out,
+      "id"_nm = id
+    });
   }
 };
 
-// [[Rcpp::export]]
-List isobands_impl(const NumericVector &x, const NumericVector &y, const NumericMatrix &z, const NumericVector &value_low, const NumericVector &value_high) {
+[[cpp11::register]]
+cpp11::writable::list isobands_impl(cpp11::doubles x, cpp11::doubles y, cpp11::doubles_matrix<> z, cpp11::doubles value_low, cpp11::doubles value_high) {
   isobander ib(x, y, z);
 
-  if (value_low.size() != value_high.size()) {
-    stop("Vectors of low and high values must have the same number of elements.");
+  int n_bands = value_low.size();
+  if (n_bands != value_high.size()) {
+    cpp11::stop("Vectors of low and high values must have the same number of elements.");
   }
 
   ib.calculate_contour();
-  List out;
+  cpp11::writable::list out;
+  out.reserve(n_bands);
 
-  auto ilo = value_low.begin();
-  for (auto ihi = value_high.begin(); ihi != value_high.end(); ) {
-    ib.set_value(*ilo, *ihi);
+  for (int i = 0; i < n_bands; ++i) {
+    ib.set_value(value_low[i], value_high[i]);
     ib.calculate_contour();
     out.push_back(ib.collect());
-    ilo++; ihi++;
   }
 
   return out;
 }
 
-// [[Rcpp::export]]
-List isolines_impl(const NumericVector &x, const NumericVector &y, const NumericMatrix &z, const NumericVector &value) {
+[[cpp11::register]]
+cpp11::writable::list isolines_impl(cpp11::doubles x, cpp11::doubles y, cpp11::doubles_matrix<> z, cpp11::doubles value) {
   isoliner il(x, y, z);
 
-  List out;
+  int n_lines = value.size();
+  cpp11::writable::list out;
+  out.reserve(n_lines);
 
-  for (auto it = value.begin(); it != value.end(); it++) {
-    il.set_value(*it);
+  for (int i = 0; i < n_lines; ++i) {
+    il.set_value(REAL(value)[i]);
     il.calculate_contour();
     out.push_back(il.collect());
   }
